@@ -134,7 +134,7 @@ class GraphTraversalSource(object):
         val = True if v is None else v
         if options_strategy is None:
             options_strategy = OptionsStrategy({k: val})
-            source = self.withStrategies(options_strategy)
+            source = self.with_strategies(options_strategy)
         else:
             options_strategy[1].configuration[k] = val
 
@@ -243,6 +243,11 @@ class GraphTraversalSource(object):
         traversal.bytecode.add_step("call", *args)
         return traversal
 
+    def union(self, *args):
+        traversal = self.get_graph_traversal()
+        traversal.bytecode.add_step("union", *args)
+        return traversal
+
 
 class GraphTraversal(Traversal):
     def __init__(self, graph, traversal_strategies, bytecode):
@@ -274,6 +279,10 @@ class GraphTraversal(Traversal):
         self.bytecode.add_step("V", *args)
         return self
 
+    def E(self, *args):
+        self.bytecode.add_step("E", *args)
+        return self
+
     def addE(self, *args):
         warnings.warn(
             "gremlin_python.process.GraphTraversal.addE will be replaced by "
@@ -366,6 +375,10 @@ class GraphTraversal(Traversal):
         self.bytecode.add_step("coin", *args)
         return self
 
+    def concat(self, *args):
+        self.bytecode.add_step("concat", *args)
+        return self
+
     def connectedComponent(self, *args):
         warnings.warn(
             "gremlin_python.process.GraphTraversalSource.connectedComponent will be replaced by "
@@ -926,6 +939,10 @@ class __(object, metaclass=MagicType):
     def __(cls, *args):
         return __.inject(*args)
 
+    @classmethod
+    def E(cls, *args):
+        return cls.graph_traversal(None, None, Bytecode()).E(*args)
+
     @classmethod
     def V(cls, *args):
         return cls.graph_traversal(None, None, Bytecode()).V(*args)
@@ -1022,6 +1039,10 @@ class __(object, metaclass=MagicType):
     def coin(cls, *args):
         return cls.graph_traversal(None, None, Bytecode()).coin(*args)
 
+    @classmethod
+    def concat(cls, *args):
+        return cls.graph_traversal(None, None, Bytecode()).concat(*args)
+
     @classmethod
     def constant(cls, *args):
         return cls.graph_traversal(None, None, Bytecode()).constant(*args)
@@ -1577,17 +1598,21 @@ class Transaction:
             raise Exception(error_message)
 
     def __close_session(self, session):
-        self._session_based_connection.close()
         self.__is_open = False
+        self._remote_connection.remove_session(self._session_based_connection)
         return session
 
 
+def E(*args):
+    return __.E(*args)
+
+
 def V(*args):
     return __.V(*args)
 
 
 def addE(*args):
-    return __.addE(*args)
+    return __.add_e(*args)
 
 
 def add_e(*args):
@@ -1595,7 +1620,7 @@ def add_e(*args):
 
 
 def addV(*args):
-    return __.addV(*args)
+    return __.add_v(*args)
 
 
 def add_v(*args):
@@ -1623,7 +1648,7 @@ def both(*args):
 
 
 def bothE(*args):
-    return __.bothE(*args)
+    return __.both_e(*args)
 
 
 def both_e(*args):
@@ -1631,7 +1656,7 @@ def both_e(*args):
 
 
 def bothV(*args):
-    return __.bothV(*args)
+    return __.both_v(*args)
 
 
 def both_v(*args):
@@ -1662,6 +1687,10 @@ def coin(*args):
     return __.coin(*args)
 
 
+def concat(*args):
+    return __.concat(*args)
+
+
 def constant(*args):
     return __.constant(*args)
 
@@ -1671,7 +1700,7 @@ def count(*args):
 
 
 def cyclicPath(*args):
-    return __.cyclicPath(*args)
+    return __.cyclic_path(*args)
 
 
 def cyclic_path(*args):
@@ -1691,7 +1720,7 @@ def element(*args):
 
 
 def elementMap(*args):
-    return __.elementMap(*args)
+    return __.element_map(*args)
 
 
 def element_map(*args):
@@ -1711,7 +1740,7 @@ def filter_(*args):
 
 
 def flatMap(*args):
-    return __.flatMap(*args)
+    return __.flat_map(*args)
 
 
 def flat_map(*args):
@@ -1727,7 +1756,7 @@ def group(*args):
 
 
 def groupCount(*args):
-    return __.groupCount(*args)
+    return __.group_count(*args)
 
 
 def group_count(*args):
@@ -1739,7 +1768,7 @@ def has(*args):
 
 
 def hasId(*args):
-    return __.hasId(*args)
+    return __.has_id(*args)
 
 
 def has_id(*args):
@@ -1747,7 +1776,7 @@ def has_id(*args):
 
 
 def hasKey(*args):
-    return __.hasKey(*args)
+    return __.has_key_(*args)
 
 
 def has_key_(*args):
@@ -1755,7 +1784,7 @@ def has_key_(*args):
 
 
 def hasLabel(*args):
-    return __.hasLabel(*args)
+    return __.has_label(*args)
 
 
 def has_label(*args):
@@ -1763,7 +1792,7 @@ def has_label(*args):
 
 
 def hasNot(*args):
-    return __.hasNot(*args)
+    return __.has_not(*args)
 
 
 def has_not(*args):
@@ -1771,7 +1800,7 @@ def has_not(*args):
 
 
 def hasValue(*args):
-    return __.hasValue(*args)
+    return __.has_value(*args)
 
 
 def has_value(*args):
@@ -1787,7 +1816,7 @@ def identity(*args):
 
 
 def inE(*args):
-    return __.inE(*args)
+    return __.in_e(*args)
 
 
 def in_e(*args):
@@ -1795,7 +1824,7 @@ def in_e(*args):
 
 
 def inV(*args):
-    return __.inV(*args)
+    return __.in_v(*args)
 
 
 def in_v(*args):
@@ -1887,7 +1916,7 @@ def order(*args):
 
 
 def otherV(*args):
-    return __.otherV(*args)
+    return __.other_v(*args)
 
 
 def other_v(*args):
@@ -1899,7 +1928,7 @@ def out(*args):
 
 
 def outE(*args):
-    return __.outE(*args)
+    return __.out_e(*args)
 
 
 def out_e(*args):
@@ -1907,7 +1936,7 @@ def out_e(*args):
 
 
 def outV(*args):
-    return __.outV(*args)
+    return __.out_v(*args)
 
 
 def out_v(*args):
@@ -1931,7 +1960,7 @@ def property(*args):
 
 
 def propertyMap(*args):
-    return __.propertyMap(*args)
+    return __.property_map(*args)
 
 
 def property_map(*args):
@@ -1959,7 +1988,7 @@ def select(*args):
 
 
 def sideEffect(*args):
-    return __.sideEffect(*args)
+    return __.side_effect(*args)
 
 
 def side_effect(*args):
@@ -1967,7 +1996,7 @@ def side_effect(*args):
 
 
 def simplePath(*args):
-    return __.simplePath(*args)
+    return __.simple_path(*args)
 
 
 def simple_path(*args):
@@ -1995,7 +2024,7 @@ def tail(*args):
 
 
 def timeLimit(*args):
-    return __.timeLimit(*args)
+    return __.time_limit(*args)
 
 
 def time_limit(*args):
@@ -2011,7 +2040,7 @@ def to(*args):
 
 
 def toE(*args):
-    return __.toE(*args)
+    return __.to_e(*args)
 
 
 def to_e(*args):
@@ -2019,7 +2048,7 @@ def to_e(*args):
 
 
 def toV(*args):
-    return __.toV(*args)
+    return __.to_v(*args)
 
 
 def to_v(*args):
@@ -2047,7 +2076,7 @@ def value(*args):
 
 
 def valueMap(*args):
-    return __.valueMap(*args)
+    return __.value_map(*args)
 
 
 def value_map(*args):
@@ -2102,6 +2131,8 @@ statics.add_static('coalesce', coalesce)
 
 statics.add_static('coin', coin)
 
+statics.add_static('concat', concat)
+
 statics.add_static('constant', constant)
 
 statics.add_static('count', count)
